@@ -88,10 +88,10 @@ cifs_read_super(struct super_block *sb, void *data, int silent)
 
 	sb->s_magic = CIFS_MAGIC_NUMBER;
 	sb->s_op = &cifs_super_ops;
-	if(cifs_sb->tcon->ses->server->maxBuf > MAX_CIFS_HDR_SIZE + 512)
-	    sb->s_blocksize = cifs_sb->tcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE;
-	else
-		sb->s_blocksize = CIFSMaximumBufferSize;
+/*	if(cifs_sb->tcon->ses->server->maxBuf > MAX_CIFS_HDR_SIZE + 512)
+	    sb->s_blocksize = cifs_sb->tcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE; */
+
+	sb->s_blocksize = CIFS_MAX_MSGSIZE;
 	sb->s_blocksize_bits = 14;	/* default 2**14 = CIFS_MAX_MSGSIZE */
 	inode = iget(sb, ROOT_I);
 
@@ -184,21 +184,22 @@ kmem_cache_t *cifs_oplock_cachep;
 static struct inode *
 cifs_alloc_inode(struct super_block *sb)
 {
-	struct cifsInodeInfo *cifs_inode;
-	cifs_inode =
-	    (struct cifsInodeInfo *) kmem_cache_alloc(cifs_inode_cachep,
-						      SLAB_KERNEL);
-	if (!cifs_inode)
-		return NULL;
-	cifs_inode->cifsAttrs = 0x20;	/* default */
-	atomic_set(&cifs_inode->inUse, 0);
-	cifs_inode->time = 0;
-	if(oplockEnabled) {
-	    cifs_inode->clientCanCacheRead = 1;
-	    cifs_inode->clientCanCacheAll = 1;
-	}
-	INIT_LIST_HEAD(&cifs_inode->openFileList);
-	return &cifs_inode->vfs_inode;
+        struct cifsInodeInfo *cifs_inode;
+        cifs_inode =
+            (struct cifsInodeInfo *) kmem_cache_alloc(cifs_inode_cachep,
+                                                      SLAB_KERNEL);
+        if (!cifs_inode)
+                return NULL;
+        cifs_inode->cifsAttrs = 0x20;   /* default */
+        atomic_set(&cifs_inode->inUse, 0);
+        cifs_inode->time = 0;
+        /* Until the file is open and we have gotten oplock
+        info back from the server, can not assume caching of
+        file data or metadata */
+        cifs_inode->clientCanCacheRead = FALSE;
+        cifs_inode->clientCanCacheAll = FALSE;
+        INIT_LIST_HEAD(&cifs_inode->openFileList);
+        return &cifs_inode->vfs_inode;
 }
 
 static void
@@ -277,6 +278,27 @@ cifs_get_sb(struct file_system_type *fs_type,
 	return sb;
 }
 
+ssize_t
+cifs_read_wrapper(struct file * file, char *read_data, size_t read_size,
+          loff_t * poffset)
+{
+	if(CIFS_I(file->f_dentry->d_inode)->clientCanCacheRead)
+		return generic_file_read(file,read_data,read_size,poffset);
+	else
+		return cifs_read(file,read_data,read_size,poffset);	
+}
+
+ssize_t
+cifs_write_wrapper(struct file * file, const char *write_data,
+           size_t write_size, loff_t * poffset) 
+{
+	if(CIFS_I(file->f_dentry->d_inode)->clientCanCacheAll)    /* check caching for write */
+		return generic_file_write(file,write_data, write_size,poffset);
+	else
+		return cifs_write(file,write_data,write_size,poffset);
+}
+
+
 static struct file_system_type cifs_fs_type = {
 	.owner = THIS_MODULE,
 	.name = "cifs",
@@ -440,6 +462,8 @@ static int cifs_oplock_thread(void * dummyarg)
 	struct list_head * tmp1;
 	struct oplock_q_entry * oplock_item;
 	struct cifsTconInfo *pTcon;
+	struct inode * inode;
+	__u16  netfid;
 	int rc = 0;
 
 	daemonize();
@@ -455,18 +479,20 @@ static int cifs_oplock_thread(void * dummyarg)
 							       qhead);
 			if(oplock_item) {
 				pTcon = oplock_item->tcon;
+				inode = oplock_item->pinode;
+				netfid = oplock_item->netfid;
 				DeleteOplockQEntry(oplock_item);
 				write_unlock(&GlobalMid_Lock);
 #if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))
-				rc = filemap_fdatasync(oplock_item->pinode->i_mapping);
+				rc = filemap_fdatasync(inode->i_mapping);
 				if(rc)
-					CIFS_I(oplock_item->pinode)->write_behind_rc 
+					CIFS_I(inode)->write_behind_rc 
 						= rc;
 #else 
-				filemap_fdatasync(oplock_item->pinode->i_mapping);
+				filemap_fdatasync(pinode->i_mapping);
 #endif
 				rc = CIFSSMBLock(0, pTcon, 
-					oplock_item->netfid,
+					netfid,
 					0 /* len */ , 0 /* offset */, 0, 
 					0, LOCKING_ANDX_OPLOCK_RELEASE,
 					0 /* wait flag */);
