@@ -155,7 +155,8 @@ struct cifsSesInfo {
 	char *serverOS;		/* name of operating system underlying the server */
 	char *serverNOS;	/* name of network operating system that the server is running */
 	char *serverDomain;	/* security realm of server */
-	int Suid;		/* needed for user level security */
+	int Suid;		/* remote smb uid  */
+	uid_t linux_uid;        /* local Linux uid */
 	int capabilities;
 	char serverName[SERVER_NAME_LEN_WITH_NULL * 2];	/* BB make bigger for tcp names - will ipv6 and sctp addresses fit here?? */
 	char userName[MAX_USERNAME_SIZE + 1];
@@ -207,6 +208,7 @@ struct cifsFileInfo {
 	/* BB add lock scope info here if needed */ ;
 	/* lock scope id (0 if none) */
 	struct file * pfile; /* needed for writepage */
+	struct inode * pInode; /* needed for oplock break */
 	int endOfSearch:1;	/* we have reached end of search */
 	int closePend:1;	/* file is marked to close */
 	int emptyDir:1;
@@ -259,8 +261,9 @@ struct mid_q_entry {
 
 struct oplock_q_entry {
 	struct list_head qhead;
-	struct file * file_to_flush;
+	struct inode * pinode;
 	struct cifsTconInfo * tcon; 
+	__u16 netfid;
 };
 
 #define   MID_FREE 0
