@@ -253,7 +253,7 @@ cifs_demultiplex_thread(struct TCP_Server_Info *server)
 						/*  iov.iov_base = smb_buffer+total_read;
 						   iov.iov_len =  pdu_length-total_read; */
 						length = sock_recvmsg(csocket, &smb_msg, 
-                                 pdu_length - total_read, 0);
+							pdu_length - total_read, 0);
          /* cERROR(1,("For iovlen %d Length received: %d with total read %d",
 						   iov.iov_len, length,total_read));       */
 						if (length == 0) {
@@ -407,7 +407,7 @@ parse_mount_options(char *options, char *devname, struct smb_vol *vol)
 				return 1;	/* needs_arg; */
 			}
 			if ((temp_len = strnlen(value, 300)) < 300) {
-				vol->UNC = kmalloc(GFP_KERNEL, temp_len);
+				vol->UNC = kmalloc(temp_len+1,GFP_KERNEL);
 				strcpy(vol->UNC,value);
 				if (strncmp(vol->UNC, "//", 2) == 0) {
 					vol->UNC[0] = '\\';
@@ -484,7 +484,7 @@ parse_mount_options(char *options, char *devname, struct smb_vol *vol)
 			return 1;
 		}
 		if ((temp_len = strnlen(devname, 300)) < 300) {
-			vol->UNC = kmalloc(GFP_KERNEL, temp_len);
+			vol->UNC = kmalloc(temp_len+1,GFP_KERNEL);
 			strcpy(vol->UNC,devname);
 			if (strncmp(vol->UNC, "//", 2) == 0) {
 				vol->UNC[0] = '\\';
@@ -862,7 +862,7 @@ cifs_mount(struct super_block *sb, struct cifs_sb_info *cifs_sb,
 		FreeXid(xid);
 		return -EINVAL;
 	}
-	/* BB add support to use the multiuser_mount flag BB */
+
 	existingCifsSes =
 	    find_tcp_session(sin_server.sin_addr.s_addr,
 			     volume_info.username, &srvTcp);
@@ -928,6 +928,7 @@ cifs_mount(struct super_block *sb, struct cifs_sb_info *cifs_sb,
 			if (volume_info.domainname)
 				strncpy(pSesInfo->domainName,
 					volume_info.domainname,MAX_USERNAME_SIZE);
+			pSesInfo->linux_uid = volume_info.linux_uid;
 
 			rc = setup_session(xid,pSesInfo, cifs_sb->local_nls);
 			if(!rc)
