@@ -49,6 +49,7 @@ cifs_open(struct inode *inode, struct file *file)
 	int desiredAccess = 0x20197;
 	int disposition = FILE_OPEN;
 	__u16 netfid;
+	FILE_ALL_INFO * buf = NULL;
 
 	xid = GetXid();
 
@@ -93,11 +94,12 @@ cifs_open(struct inode *inode, struct file *file)
 
 	/* BB pass O_SYNC flag through on file attributes .. BB */
 
-	/* BB add code to refresh inode by passing in file_info buf on open 
+	/* Also refresh inode by passing in file_info buf returned by SMBOpen 
 	   and calling get_inode_info with returned buf (at least 
 	   helps non-Unix server case */
+        buf = kmalloc(sizeof(FILE_ALL_INFO),GFP_KERNEL);
 	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
-			CREATE_NOT_DIR, &netfid, &oplock, NULL, cifs_sb->local_nls);
+			CREATE_NOT_DIR, &netfid, &oplock, buf, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("cifs_open returned 0x%x ", rc));
 		cFYI(1, ("oplock: %d ", oplock));	
@@ -116,8 +118,22 @@ cifs_open(struct inode *inode, struct file *file)
 			write_lock(&GlobalSMBSeslock);
 			list_add(&pCifsFile->tlist,&pTcon->openFileList);
 			pCifsInode = CIFS_I(file->f_dentry->d_inode);
-			if(pCifsInode->openFileList.next)
+			if(pCifsInode) {
+		                if (pTcon->ses->capabilities & CAP_UNIX)
+					rc = cifs_get_inode_info_unix(&file->f_dentry->d_inode,
+						full_path, inode->i_sb);
+				else
+					rc = cifs_get_inode_info(&file->f_dentry->d_inode,
+						full_path, buf, inode->i_sb);
+
 				list_add(&pCifsFile->flist,&pCifsInode->openFileList);
+				if(oplock == OPLOCK_EXCLUSIVE) {
+					pCifsInode->clientCanCacheAll = TRUE;
+					pCifsInode->clientCanCacheRead = TRUE;
+					cFYI(1,("Exclusive Oplock granted on inode %p",file->f_dentry->d_inode));
+				} else if(oplock == OPLOCK_READ)
+					pCifsInode->clientCanCacheRead = TRUE;
+			}
 			write_unlock(&GlobalSMBSeslock);
 			spin_unlock(&files_lock);
 			if(file->f_flags & O_CREAT) {           
@@ -138,6 +154,8 @@ cifs_open(struct inode *inode, struct file *file)
 		}
 	}
 
+	if (buf)
+		kfree(buf);
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
@@ -221,6 +239,13 @@ cifs_close(struct inode *inode, struct file *file)
 	} else
 		rc = -EBADF;
 
+	if(list_empty(&(CIFS_I(inode)->openFileList))) {
+		cFYI(1,("closing last open instance for inode %p",inode));
+		/* if the file is not open we do not know if we can cache
+		info on this inode, much less write behind and read ahead */
+		CIFS_I(inode)->clientCanCacheRead = FALSE;
+		CIFS_I(inode)->clientCanCacheAll  = FALSE;
+	}
 	if((rc ==0) && CIFS_I(inode)->write_behind_rc)
 		rc = CIFS_I(inode)->write_behind_rc;
 	FreeXid(xid);
@@ -462,8 +487,8 @@ cifs_partialpagewrite(struct page *page,unsigned from, unsigned to)
 	cifsInode = CIFS_I(mapping->host);
 	read_lock(&GlobalSMBSeslock);
 	list_for_each_safe(tmp, tmp1, &cifsInode->openFileList) {            
-		open_file = list_entry(tmp,struct cifsFileInfo, flist);           
-	/* We could check if file is open for writing first */
+		open_file = list_entry(tmp,struct cifsFileInfo, flist);
+		/* We check if file is open for writing first */
 		if((open_file->pfile) && 
 		   ((open_file->pfile->f_flags & O_RDWR) || 
 			(open_file->pfile->f_flags & O_WRONLY))) {
@@ -818,7 +843,13 @@ cifs_readpages(struct file *file, struct address_space *mapping,
 				break;
 			}
 		} else {
-			cFYI(1,("No bytes read"));
+			cFYI(1,("No bytes read cleaning remaining pages off readahead list"));
+			/* BB turn off caching and do new lookup on file size at server? */
+			while (!list_empty(page_list) && (i < num_pages)) {
+				page = list_entry(page_list->prev, struct page, list);
+				list_del(&page->list);
+			}
+
 			break;
 		}
 		if(smb_read_data) {
@@ -936,7 +967,7 @@ fill_in_inode(struct inode *tmp_inode,
 	/* can not fill in nlink here as in qpathinfo version and Unx search */
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
-	    do_div(pfindData->AllocationSize, tmp_inode->i_blksize);
+		(tmp_inode->i_blksize - 1 + pfindData->AllocationSize) >> tmp_inode->i_blkbits;
 	if (pfindData->AllocationSize < pfindData->EndOfFile)
 		cFYI(1, ("Possible sparse file: allocation size less than end of file "));
 	cFYI(1,
@@ -1010,7 +1041,8 @@ unix_fill_in_inode(struct inode *tmp_inode,
 	pfindData->EndOfFile = le64_to_cpu(pfindData->EndOfFile);
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
-	    do_div(pfindData->NumOfBytes, tmp_inode->i_blksize);
+                (tmp_inode->i_blksize - 1 + pfindData->NumOfBytes) >> tmp_inode->i_blkbits;
+
 	if (S_ISREG(tmp_inode->i_mode)) {
 		cFYI(1, (" File inode "));
 		tmp_inode->i_op = &cifs_file_inode_ops;
@@ -1038,6 +1070,7 @@ construct_dentry(struct qstr *qstring, struct file *file,
 	struct dentry *tmp_dentry;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
+	struct cifsInodeInfo *pCifsI;
 
 	cFYI(1, ("For %s ", qstring->name));
 	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
@@ -1049,9 +1082,45 @@ construct_dentry(struct qstr *qstring, struct file *file,
 		cFYI(0, (" existing dentry with inode 0x%p", tmp_dentry->d_inode));
 		*ptmp_inode = tmp_dentry->d_inode;
 		/* BB overwrite the old name? i.e. tmp_dentry->d_name and tmp_dentry->d_name.len ?? */
+		if(*ptmp_inode == NULL) {
+	                *ptmp_inode = new_inode(file->f_dentry->d_sb);
+			d_instantiate(tmp_dentry, *ptmp_inode);
+                        pCifsI = CIFS_I(*ptmp_inode);
+                        INIT_LIST_HEAD(&pCifsI->openFileList);
+                        /* can not enable caching for this inode
+                        until a file instance is open and we
+                        can check the oplock flag on the open
+                        response */
+                        (*ptmp_inode)->i_blksize = CIFS_MAX_MSGSIZE;
+                        (*ptmp_inode)->i_blkbits = 14;  /* 2**14 = CIFS_MAX_MSGSIZE */
+                        pCifsI->clientCanCacheRead = FALSE;
+                        pCifsI->clientCanCacheAll = FALSE;
+                        pCifsI->time = 0;
+                        /* do not need to set cifs Attrs since
+                                they are about to be overwritten
+                                in fill_in_inode */
+                        atomic_set(&pCifsI->inUse, 0);
+		}
 	} else {
 		tmp_dentry = d_alloc(file->f_dentry, qstring);
 		*ptmp_inode = new_inode(file->f_dentry->d_sb);
+		if(ptmp_inode) {
+			pCifsI = CIFS_I(*ptmp_inode);
+			INIT_LIST_HEAD(&pCifsI->openFileList);
+			/* can not enable caching for this inode 
+			until a file instance is open and we
+			can check the oplock flag on the open
+			response */
+	                (*ptmp_inode)->i_blksize = CIFS_MAX_MSGSIZE;
+			(*ptmp_inode)->i_blkbits = 14;  /* 2**14 = CIFS_MAX_MSGSIZE */
+			pCifsI->clientCanCacheRead = FALSE;
+			pCifsI->clientCanCacheAll = FALSE;
+			pCifsI->time = 0;
+			/* do not need to set cifs Attrs since
+				they are about to be overwritten 
+				in fill_in_inode */
+			atomic_set(&pCifsI->inUse, 0);
+		}
 		tmp_dentry->d_op = &cifs_dentry_ops;
 		cFYI(0, (" instantiate dentry 0x%p with inode 0x%p ",
 			 tmp_dentry, *ptmp_inode));
@@ -1202,7 +1271,7 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 			renew_parental_timestamps(file->f_dentry);
 			lastFindData = 
 				(FILE_DIRECTORY_INFO *) ((char *) pfindData + 
-					le32_to_cpu(findParms.LastNameOffset));
+					findParms.LastNameOffset);
 			if((char *)lastFindData > (char *)pfindData + bufsize) {
 				cFYI(1,("last search entry past end of packet"));
 				rc = -EIO;
@@ -1362,7 +1431,7 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 			/* BB save off resume key, key name and name length  */
 				lastFindData = 
 					(FILE_DIRECTORY_INFO *) ((char *) pfindData 
-						+ le32_to_cpu(findNextParms.LastNameOffset));
+						+ findNextParms.LastNameOffset);
 				if((char *)lastFindData > (char *)pfindData + bufsize) {
 					cFYI(1,("last search entry past end of packet"));
 					rc = -EIO;
