@@ -36,10 +36,8 @@ struct inode * get_cifs_inode(struct super_block * sb)
 	cFYI(1,("got new inode %p",newinode));
 	if(newinode) {    
 		struct cifsInodeInfo * cifsInfo = CIFS_I(newinode);
-		if(oplockEnabled) {
-			cifsInfo->clientCanCacheRead = 1;
-			cifsInfo->clientCanCacheAll = 1;
-		}
+		cifsInfo->clientCanCacheRead = FALSE;
+		cifsInfo->clientCanCacheAll = FALSE;
 		INIT_LIST_HEAD(&cifsInfo->openFileList);
 		cifsInfo->cifsAttrs = 0x20;   /* default */
 		atomic_set(&cifsInfo->inUse, 0);
@@ -62,6 +60,9 @@ cifs_get_inode_info_unix(struct inode **pinode,
 	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
 	char *tmp_path;
 
+/* BB add caching check so we do not go to server to overwrite inode info to cached file
+	where the local file sizes are correct and the server info is stale  BB */
+
 	xid = GetXid();
 
 	pTcon = cifs_sb->tcon;
@@ -142,9 +143,13 @@ cifs_get_inode_info_unix(struct inode **pinode,
 		findData.NumOfBytes = le64_to_cpu(findData.NumOfBytes);
 		findData.EndOfFile = le64_to_cpu(findData.EndOfFile);
 		inode->i_size = findData.EndOfFile;
-		inode->i_blksize =
-		    (pTcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
-		inode->i_blocks = do_div(findData.NumOfBytes, inode->i_blksize);
+/* blksize needs to be multiple of two. So safer to default to blksize
+	and blkbits set in superblock so 2**blkbits and blksize will match */
+/*		inode->i_blksize =
+		    (pTcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;*/
+		inode->i_blocks = 
+	                (inode->i_blksize - 1 + findData.NumOfBytes) >> inode->i_blkbits;
+
 		if (findData.NumOfBytes < findData.EndOfFile)
 			cFYI(1, ("Server inconsistency Error: it says allocation size less than end of file "));
 		cFYI(1,
@@ -188,7 +193,15 @@ cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path,
 	xid = GetXid();
 
 	pTcon = cifs_sb->tcon;
-	cFYI(1, (" Getting info on %s ", search_path));
+	cFYI(1,("Getting info on %s ", search_path));
+
+	if((pfindData == NULL) && (*pinode != NULL)) {
+		if(CIFS_I(*pinode)->clientCanCacheRead) {
+			cFYI(1,("No need to revalidate inode sizes on cached file "));
+			FreeXid(xid);
+			return rc;
+		}
+	}
 
 	/* if file info not passed in then get it from server */
 	if(pfindData == NULL) {
@@ -244,8 +257,11 @@ cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path,
 		cFYI(1, (" New time %ld ", cifsInfo->time));
 		atomic_inc(&cifsInfo->inUse);	/* inc on every refresh of inode */
 
-		inode->i_blksize =
-		    (pTcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
+/* blksize needs to be multiple of two. So safer to default to blksize
+        and blkbits set in superblock so 2**blkbits and blksize will match */
+/*		inode->i_blksize =
+		    (pTcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;*/
+
 		/* Linux can not store file creation time unfortunately so we ignore it */
 		inode->i_atime =
 		    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastAccessTime));
@@ -276,7 +292,8 @@ cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path,
 		inode->i_size = le64_to_cpu(pfindData->EndOfFile);
 		pfindData->AllocationSize = le64_to_cpu(pfindData->AllocationSize);
 		inode->i_blocks =
-		    do_div(pfindData->AllocationSize, inode->i_blksize);
+	                (inode->i_blksize - 1 + pfindData->AllocationSize) >> inode->i_blkbits;
+
 		cFYI(1,
 		     (" Size %ld and blocks %ld ",
 		      (unsigned long) inode->i_size, inode->i_blocks));
